#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <UniversalTelegramBot.h>
#include <ThingSpeak.h>
#include <DHT.h>

//DHT 22
#define pinDHT22 2
DHT dht22(pinDHT22, DHT22);

// Wifi network station credentials
#define ssid "NAMAWIFI"
#define pass "PASSWORDWIFI"

// Initialize Telegram BOT
#define BOTtoken "TOKENBOT"  // your Bot Token (Get from Botfather)
#define chatID "IDTELEGRAM"
const unsigned long BOT_MTBS = 1000;  // mean time between scan messages
unsigned long bot_lasttime;

X509List cert(TELEGRAM_CERTIFICATE_ROOT);
WiFiClientSecure secured_client;
UniversalTelegramBot bot(BOTtoken, secured_client);
WiFiClient client;

//Time Thingspeak
unsigned long time1 = 0;
unsigned long time2 = 0;
const unsigned long updateInterval1 = 1800L * 1000L;    // Post data every 30 minutes at channel 1 
const unsigned long updateInterval2 = 3600L * 1000L;  // Post data every 1 hour at channel 2 

//ThingSpeak Channel 1 
unsigned long channel1 = XXXXXX;                 // Channel ID
const char *myWriteAPIKey1 = "WRITEAPIKEY";  // ThingSpeak Channel API key
const char *myReadAPIKey1 = "READAPIKEY";   // Read API Key

//ThingSpeak Channel 2 
unsigned long channel2 = 2752238;                 // Channel ID
const char *myWriteAPIKey2 = "WRITEAPIKEY";  // ThingSpeak Channel API key
const char *myReadAPIKey2 = "READAPIKEY";   // Read API Key

//Respon telegram ketika mendapatkan pesan
void handleNewMessages(int numNewMessages) {

  Serial.print("handleNewMessages ");
  Serial.println(numNewMessages);

  float t = dht22.readTemperature();
  int h = dht22.readHumidity();
  float t1 = ThingSpeak.readFloatField(channel1, 1, myReadAPIKey1);
  int h1 = ThingSpeak.readFloatField(channel1, 2, myReadAPIKey1);
  float t2 = ThingSpeak.readFloatField(channel2, 1, myReadAPIKey2);
  int h2 = ThingSpeak.readFloatField(channel2, 2, myReadAPIKey2);

  //Info real time
  String realtime = "Real time \n" "Suhu: " + String(t, 1) + "°C \n" "Kelembaban: "  + String(h) + " % \n";
  //Info setengah jam
  String setengahjam = "Setengah jam terakhir \n" "Suhu: " + String(t1, 1) + "°C \n" "Kelembaban: " + String(h1) + " % \n";
  //Info satu jam
  String info= "Suhu: " + String(t2, 1) + "°C \n" "Kelembaban: " + String(h2) + " % \n";

  for (int i = 0; i < numNewMessages; i++) {
    telegramMessage &msg = bot.messages[i];
    Serial.println("Received " + msg.text);
    //info realtime
    if (msg.text == "/realtimestg") {
      if (isnan(t) || isnan(h)) {
        Serial.println("Failed to read from DHT22 sensor!");
        bot.sendMessage(chatID, "Sensor tidak terbaca!", "");
        delay(600000);
      }
      else {
        bot.sendMessage(msg.chat_id, realtime);
      }
    }
    //info setengah jam terakhir
    else if (msg.text == "/setengahjamstg") {
      bot.sendMessage(msg.chat_id, setengahjam);
    }
    //info satu jam terakhir
    else if (msg.text == "/info") {
      bot.sendMessage(msg.chat_id, info);
    } else {
      bot.sendMessage(msg.chat_id, "silahkan pilih /info untuk mendapatkan informasi suhu dan kelembapan 1 jam terakhir \n ");
    }
  }
}

//Telegram bot update (mengirimkan pesan)
void botUpdates(){

  if (millis() - bot_lasttime > BOT_MTBS) {
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    while (numNewMessages) {
      Serial.println("got response");
      handleNewMessages(numNewMessages);
      numNewMessages = bot.getUpdates(bot.last_message_received + 1);
    }

    bot_lasttime = millis();
  }
}

//Mengirim pesan ke ThingSpeak
void thingSpeak(){
    
  float t = dht22.readTemperature();
  int h = dht22.readHumidity();
  
  if (millis() - time1 >= updateInterval1){  //Interval 30 menit - Private
    time1 = millis();
    if (isnan(t) || isnan(h)) {
      Serial.println("Failed to read from DHT22 sensor!");
      bot.sendMessage(chatID, "Sensor tidak terbaca!", "");
      delay(600000);
    } else {
      ThingSpeak.setField(1, t);
      ThingSpeak.setField(2, h);
      int x = ThingSpeak.writeFields(channel1, myWriteAPIKey1);
      Serial.print(" DHT22 | Suhu: ");
      Serial.print(t, 1);
      Serial.print("°C ");
      Serial.print("  |  ");
      Serial.print("Kelembaban: ");
      Serial.print(h);
      Serial.println("% ");
      if (millis() - time2 >= updateInterval2){   //Interval 1 jam - Public
        time2 = millis();
        ThingSpeak.setField(1, t);
        ThingSpeak.setField(2, h);
        int y = ThingSpeak.writeFields(channel2, myWriteAPIKey2);
      }
    }
  }
}

void setup() {
  Serial.begin(115200);
  dht22.begin();  // initialize the DHT22 sensor

  //Attempt to connect to Wifi network:
  configTime(0, 0, "pool.ntp.org");       // get UTC time via NTP
  secured_client.setTrustAnchors(&cert);  // Add root certificate for api.telegram.org
  WiFi.begin(ssid, pass);
  Serial.print("Connecting Wifi: ");
  Serial.println(ssid);

  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(1000);
  }

  Serial.print("\nWiFi connected. IP address: ");
  Serial.println(WiFi.localIP());

  bot.sendMessage(chatID, "Bot mulai", "");

  ThingSpeak.begin(client);  // menjadikan kita client ThingSpeak
}

void loop() {

  thingSpeak();
  botUpdates();
}